@@ -1,5 +1,5 @@
 //! `Tensor`, tract main data object of interest.
-use crate::datum::{scale_by, Blob, ClampCast, Datum, DatumType};
+use crate::datum::{round_ties_to_even, scale_by, Blob, ClampCast, Datum, DatumType, QParams};
 use crate::dim::TDim;
 use crate::f16::f16;
 use crate::TVec;
@@ -868,7 +868,7 @@ impl Tensor {
                 }
 
                 macro_rules! q_f {
-                    ($source:ty, $dest:ty) => {
+                    ($source:ty, $dest:ty, $round:expr) => {
                         if <$source>::datum_type().unquantized() == self.datum_type().unquantized()
                             && <$dest>::datum_type().unquantized() == dst_dt.unquantized()
                         {
@@ -878,7 +878,7 @@ impl Tensor {
                                 .for_each(|(&s, d)| {
                                     let s_float = (s as f32 - s_zp as f32) * s_scale as f32;
                                     let d_float = s_float as f32 / d_scale as f32 + d_zp as f32;
-                                    *d = d_float.clamp_cast();
+                                    *d = $round(d_float).clamp_cast();
                                 });
                             return Ok(Cow::Owned(result));
                         }
@@ -886,7 +886,7 @@ impl Tensor {
                 }
 
                 macro_rules! q_n {
-                    ($source:ty, $dest:ty) => {{
+                    (clamp $source:ty, $dest:ty) => {{
                         if <$source>::datum_type().unquantized() == self.datum_type().unquantized()
                             && <$dest>::datum_type().unquantized() == dst_dt.unquantized()
                         {
@@ -899,6 +899,19 @@ impl Tensor {
                             return Ok(Cow::Owned(result));
                         }
                     }};
+                    ($source:ty, $dest:ty) => {{
+                        if <$source>::datum_type().unquantized() == self.datum_type().unquantized()
+                            && <$dest>::datum_type().unquantized() == dst_dt.unquantized()
+                        {
+                            self.as_slice_unchecked::<$source>()
+                                .iter()
+                                .zip(result.as_slice_mut_unchecked::<$dest>().iter_mut())
+                                .for_each(|(&s, d)| {
+                                    *d = s as $dest;
+                                });
+                            return Ok(Cow::Owned(result));
+                        }
+                    }};
                 }
 
                 if dst_dt.unquantized() == self.datum_type().unquantized()
@@ -909,24 +922,24 @@ impl Tensor {
                     q8_to_q8!(u8);
                 }
 
-                q_f!(f32, i8);
-                q_f!(f32, u8);
-                q_f!(i8, f32);
-                q_f!(u8, f32);
+                q_f!(f32, i8, round_ties_to_even);
+                q_f!(f32, u8, round_ties_to_even);
+                q_f!(i8, f32, |f| f);
+                q_f!(u8, f32, |f| f);
 
                 if dst_dt.is_quantized() && self.datum_type().is_quantized() {
-                    q_f!(u8, i8);
-                    q_f!(i8, u8);
+                    q_f!(u8, i8, round_ties_to_even);
+                    q_f!(i8, u8, round_ties_to_even);
                 }
 
                 q_n!(i8, i32);
                 q_n!(i8, u32);
                 q_n!(u8, i32);
                 q_n!(u8, u32);
-                q_n!(i32, i8);
-                q_n!(i32, u8);
-                q_n!(u32, i8);
-                q_n!(u32, u8);
+                q_n!(clamp i32, i8);
+                q_n!(clamp i32, u8);
+                q_n!(clamp u32, i8);
+                q_n!(clamp u32, u8);
                 q_n!(i8, i8);
                 q_n!(u8, u8);
                 q_n!(i32, i32);
@@ -1086,6 +1099,25 @@ impl Tensor {
     pub fn view_at_prefix_mut(&mut self, prefix: &[usize]) -> anyhow::Result<view::TensorView> {
         view::TensorView::at_prefix(self, prefix)
     }
+
+    /// Offsets the tensor as an i8 type if it's an u8 type, otherwise passes it unchanged.
+    pub fn offset_u8_as_i8(self: &Arc<Self>) -> Arc<Self> {
+        let mut t = if let DatumType::U8 = self.dt.unquantized() {
+            self.to_array_view::<u8>().unwrap().mapv(|v| v.wrapping_sub(128) as i8).into_tensor()
+        } else {
+            return self.clone();
+        };
+
+        if let DatumType::QU8(qp) = self.dt {
+            if let QParams::ZpScale { zero_point, scale } = qp {
+                t.dt = DatumType::QI8(QParams::ZpScale { zero_point: zero_point - 128, scale });
+            } else {
+                t.dt = DatumType::QI8(qp);
+            }
+        }
+
+        t.into_arc_tensor()
+    }
 }
 
 impl PartialEq for Tensor {
